/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.appletgui;

import jalview.viewmodel.OverviewDimensionsWithHidden;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Panel;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

public class OverviewPanel extends Panel implements Runnable,
        MouseMotionListener, MouseListener
{
  private OverviewDimensionsWithHidden od;

  private OverviewCanvas oviewCanvas;

  private AlignViewport av;

  private AlignmentPanel ap;

  private boolean updateRunning = false;

  public OverviewPanel(AlignmentPanel alPanel)
  {
    this.av = alPanel.av;
    this.ap = alPanel;
    setLayout(null);

    od = new OverviewDimensionsWithHidden(av.getRanges(),
            (av.isShowAnnotation() && av.getSequenceConsensusHash() != null));

    oviewCanvas = new OverviewCanvas(od, av);
    setLayout(new BorderLayout());
    add(oviewCanvas, BorderLayout.CENTER);

    setSize(new Dimension(od.getWidth(), od.getHeight()));
    addComponentListener(new ComponentAdapter()
    {

      @Override
      public void componentResized(ComponentEvent evt)
      {
        if ((getWidth() != od.getWidth())
                || (getHeight() != (od.getHeight())))
        {
          updateOverviewImage();
        }
      }
    });

    addMouseMotionListener(this);

    addMouseListener(this);

    updateOverviewImage();

  }

  @Override
  public void mouseEntered(MouseEvent evt)
  {
  }

  @Override
  public void mouseExited(MouseEvent evt)
  {
  }

  @Override
  public void mouseClicked(MouseEvent evt)
  {
  }

  @Override
  public void mouseMoved(MouseEvent evt)
  {
  }

  @Override
  public void mousePressed(MouseEvent evt)
  {
    mouseAction(evt);
  }

  @Override
  public void mouseReleased(MouseEvent evt)
  {
    mouseAction(evt);
  }

  @Override
  public void mouseDragged(MouseEvent evt)
  {
    mouseAction(evt);
  }

  private void mouseAction(MouseEvent evt)
  {
    od.updateViewportFromMouse(evt.getX(), evt.getY(), av.getAlignment()
            .getHiddenSequences(), av.getColumnSelection(), av
            .getRanges());
    ap.setScrollValues(od.getScrollCol(), od.getScrollRow());
    ap.paintAlignment(false);
  }

  /**
   * Updates the overview image when the related alignment panel is updated
   */
  public void updateOverviewImage()
  {
    if ((getSize().width > 0) && (getSize().height > 0))
    {
      od.setWidth(getSize().width);
      od.setHeight(getSize().height);
    }
    setSize(new Dimension(od.getWidth(), od.getHeight()));

    if (updateRunning)
    {
      oviewCanvas.restartDraw();
      return;
    }

    updateRunning = true;
    Thread thread = new Thread(this);
    thread.start();
    repaint();
    updateRunning = false;
  }

  @Override
  public void run()
  {
    oviewCanvas.draw(av.isShowSequenceFeatures(),
            (av.isShowAnnotation() && av
                    .getAlignmentConservationAnnotation() != null), ap);
    setBoxPosition();
  }

  /**
   * Update the overview panel box when the associated alignment panel is
   * changed
   * 
   */
  public void setBoxPosition()
  {
    od.setBoxPosition(av.getAlignment()
            .getHiddenSequences(), av.getColumnSelection(), av.getRanges());
    repaint();
  }
}
